@@ -1,16 +1,26 @@
 import functools
-from typing import Any, Literal, Type
+from typing import Any, Dict, Type
 
 from mongoengine import Document, EmbeddedDocument, fields
+from mongoengine.base import TopLevelDocumentMetaclass
 
-from aggify.compiler import F, Match, Q  # noqa keep
-from aggify.exceptions import AggifyValueError, AnnotationError, InvalidField, InvalidEmbeddedField, OutStageError
+from aggify.compiler import F, Match, Q, Operators, Cond  # noqa keep
+from aggify.exceptions import (
+    AggifyValueError,
+    AnnotationError,
+    InvalidField,
+    InvalidEmbeddedField,
+    OutStageError,
+    InvalidArgument,
+)
 from aggify.types import QueryParams
 from aggify.utilty import (
     to_mongo_positive_index,
     check_fields_exist,
     replace_values_recursive,
     convert_match_query,
+    check_field_exists,
+    get_db_field,
 )
 
 
@@ -54,15 +64,18 @@ class Aggify:
         return self
 
     @last_out_stage_check
-    def group(self, key: str = "_id") -> "Aggify":
-        self.pipelines.append({"$group": {"_id": f"${key}"}})
+    def group(self, expression: str | None = "_id") -> "Aggify":
+        expression = f"${expression}" if expression else None
+        self.pipelines.append({"$group": {"_id": expression}})
         return self
 
     @last_out_stage_check
-    def order_by(self, field: str) -> "Aggify":
-        self.pipelines.append(
-            {"$sort": {f'{field.replace("-", "")}': -1 if field.startswith("-") else 1}}
-        )
+    def order_by(self, *fields: str | list[str]) -> "Aggify":
+        sort_dict = {
+            field.replace("-", ""): -1 if field.startswith("-") else 1
+            for field in fields
+        }
+        self.pipelines.append({"$sort": sort_dict})
         return self
 
     @last_out_stage_check
@@ -71,7 +84,7 @@ class Aggify:
         return self
 
     @last_out_stage_check
-    def add_fields(self, fields: dict) -> "Aggify":  # noqa
+    def add_fields(self, **fields) -> "Aggify":  # noqa
         """
         Generates a MongoDB addFields pipeline stage.
 
@@ -88,6 +101,8 @@ class Aggify:
                 add_fields_stage["$addFields"][field] = {"$literal": expression}
             elif isinstance(expression, F):
                 add_fields_stage["$addFields"][field] = expression.to_dict()
+            elif isinstance(expression, Cond):
+                add_fields_stage["$addFields"][field] = dict(expression)
             else:
                 raise AggifyValueError([str, F], type(expression))
 
@@ -163,19 +178,33 @@ class Aggify:
             if key in skip_list:
                 continue
 
+            # Split the key to access the field information.
             split_query = key.split("__")
-            join_field = self.base_model._fields.get(split_query[0])  # type: ignore
-            if not join_field:
-                raise ValueError(f"Invalid field: {split_query[0]}")
-            # This is a nested query.
-            if "document_type_obj" not in join_field.__dict__ or issubclass(
-                join_field.document_type, EmbeddedDocument
+
+            # Retrieve the field definition from the model.
+            join_field = self.get_model_field(self.base_model, split_query[0])  # type: ignore
+            # Check conditions for creating a 'match' pipeline stage.
+            if (
+                isinstance(
+                    join_field, TopLevelDocumentMetaclass
+                )  # check whether field is added by lookup stage or not
+                or "document_type_obj"
+                not in join_field.__dict__  # Check whether this field is a join field or not.
+                or issubclass(
+                    join_field.document_type, EmbeddedDocument
+                )  # Check whether this field is embedded field or not
+                or len(split_query) == 1
+                or (len(split_query) == 2 and split_query[1] in Operators.ALL_OPERATORS)
             ):
+                # Create a 'match' pipeline stage.
                 match = self.__match({key: value})
-                if (match.get("$match")) != {}:
+
+                # Check if the 'match' stage is not empty before adding it to the pipelines.
+                if match.get("$match"):
                     self.pipelines.append(match)
+
             else:
-                from_collection = join_field.document_type._meta["collection"]  # noqa
+                from_collection = join_field.document_type
                 local_field = join_field.db_field
                 as_name = join_field.name
                 matches = []
@@ -187,18 +216,17 @@ class Aggify:
                         if match != {}:
                             matches.append(match)
 
-                self.pipelines.extend(
-                    [
-                        self.__lookup(
-                            from_collection=from_collection,
-                            local_field=local_field,
-                            as_name=as_name,
-                        ),
-                        self.unwind(as_name),  # type: ignore
-                        *[{"$match": match} for match in matches],
-                    ]
+                self.pipelines.append(
+                    self.__lookup(
+                        from_collection=from_collection._meta["collection"],  # noqa
+                        local_field=local_field,
+                        as_name=as_name,
+                    )
                 )
+                self.unwind(as_name, preserve=True)
+                self.pipelines.extend([{"$match": match} for match in matches])
 
+    @last_out_stage_check
     def __getitem__(self, index: slice | int) -> "Aggify":
         """
         # TODO: missing docs
@@ -211,24 +239,55 @@ class Aggify:
         self.pipelines.append({"$limit": int(index.stop - index.start)})
         return self
 
-    @staticmethod
+    @last_out_stage_check
     def unwind(
-        path: str, preserve: bool = True
-    ) -> dict[
-        Literal["$unwind"],
-        dict[Literal["path", "preserveNullAndEmptyArrays"], str | bool],
-    ]:
-        """
-        Generates a MongoDB unwind pipeline stage.
+        self, path: str, include_index_array: str | None = None, preserve: bool = False
+    ) -> "Aggify":
+        """Generates a MongoDB unwind pipeline stage.
 
         Args:
-            path: The path to unwind.
-            preserve: Whether to preserve null and empty arrays.
+            path: Field path to an array field.
+              To specify a field path, prefix the field name with a dollar sign $
+              and enclose in quotes.
 
-        Returns:
-            A MongoDB unwind pipeline stage.
+            include_index_array: The name of a new field to hold the array index of the element.
+              The name cannot start with a dollar sign $.
+
+            preserve: Whether to preserve null and empty arrays.
+                  If true, if the path is null, missing, or an empty array,
+                  $unwind outputs the document.
+
+                  If false, if path is null, missing, or an empty array,
+                  $unwind does not output a document.
+
+        Stages:
+            { $unwind: <field path> }
+
+            {
+              $unwind:
+                {
+                  path: <field path>,
+                  includeArrayIndex: <string>,
+                  preserveNullAndEmptyArrays: <boolean>
+                }
+            }
+
+        docs: https://www.mongodb.com/docs/manual/reference/operator/aggregation/unwind/
         """
-        return {"$unwind": {"path": f"${path}", "preserveNullAndEmptyArrays": preserve}}
+
+        if include_index_array is None and preserve is False:
+            self.pipelines.append({"$unwind": f"${path}"})
+            return self
+        self.pipelines.append(
+            {
+                "$unwind": {
+                    "path": f"${path}",
+                    "includeArrayIndex": include_index_array,
+                    "preserveNullAndEmptyArrays": preserve,
+                }
+            }
+        )
+        return self
 
     def aggregate(self):
         """
@@ -239,49 +298,85 @@ class Aggify:
         """
         return self.base_model.objects.aggregate(*self.pipelines)  # type: ignore
 
-    def annotate(self, annotate_name, accumulator, f):
-        try:
-            if (stage := list(self.pipelines[-1].keys())[0]) != "$group":
-                raise AnnotationError(
-                    f"Annotations apply only to $group, not to {stage}."
-                )
+    def annotate(
+        self, annotate_name: str, accumulator: str, f: str | dict | F | int
+    ) -> "Aggify":
+        """
+        Annotate a MongoDB aggregation pipeline with a new field.
+        Usage: https://www.mongodb.com/docs/manual/reference/operator/aggregation/group/#accumulator-operator
 
-        except IndexError as error:
-            raise AnnotationError(
-                "Annotations apply only to $group, you're pipeline is empty."
-            ) from error
+        Args:
+            annotate_name (str): The name of the new annotated field.
+            accumulator (str): The aggregation accumulator operator (e.g., "$sum", "$avg").
+            f (str | dict | F | int): The value for the annotated field.
 
-        accumulator_dict = {
-            "sum": "$sum",
-            "avg": "$avg",
-            "first": "$first",
-            "last": "$last",
-            "max": "$max",
-            "min": "$min",
-            "push": "$push",
-            "addToSet": "$addToSet",
-            "stdDevPop": "$stdDevPop",
-            "stdDevSamp": "$stdDevSamp",
-            "merge": "$mergeObjects",
+        Returns:
+            self.
+
+        Raises:
+            AnnotationError: If the pipeline is empty or if an invalid accumulator is provided.
+
+        Example:
+            annotate("totalSales", "sum", "sales")
+        """
+
+        # Some of the accumulator fields might be false and should be checked.
+        aggregation_mapping: Dict[str, Type] = {
+            "sum": (fields.FloatField(), "$sum"),
+            "avg": (fields.FloatField(), "$avg"),
+            "stdDevPop": (fields.FloatField(), "$stdDevPop"),
+            "stdDevSamp": (fields.FloatField(), "$stdDevSamp"),
+            "push": (fields.ListField(), "$push"),
+            "addToSet": (fields.ListField(), "$addToSet"),
+            "count": (fields.IntField(), "$count"),
+            "first": (fields.EmbeddedDocumentField(fields.EmbeddedDocument), "$first"),
+            "last": (fields.EmbeddedDocumentField(fields.EmbeddedDocument), "$last"),
+            "max": (fields.DynamicField(), "$max"),
+            "accumulator": (fields.DynamicField(), "$accumulator"),
+            "min": (fields.DynamicField(), "$min"),
+            "median": (fields.DynamicField(), "$median"),
+            "mergeObjects": (fields.DictField(), "$mergeObjects"),
+            "top": (fields.EmbeddedDocumentField(fields.EmbeddedDocument), "$top"),
+            "bottom": (
+                fields.EmbeddedDocumentField(fields.EmbeddedDocument),
+                "$bottom",
+            ),
+            "topN": (fields.ListField(), "$topN"),
+            "bottomN": (fields.ListField(), "$bottomN"),
+            "firstN": (fields.ListField(), "$firstN"),
+            "lastN": (fields.ListField(), "$lastN"),
+            "maxN": (fields.ListField(), "$maxN"),
         }
 
-        # Determine the data type based on the accumulator
-        if accumulator in ["sum", "avg", "stdDevPop", "stdDevSamp"]:
-            field_type = fields.FloatField()
-        elif accumulator in ["push", "addToSet"]:
-            field_type = fields.ListField()
-        else:
-            field_type = fields.StringField()
+        try:
+            stage = list(self.pipelines[-1].keys())[0]
+            if stage != "$group":
+                raise AnnotationError(
+                    f"Annotations apply only to $group, not to {stage}"
+                )
+        except IndexError:
+            raise AnnotationError(
+                "Annotations apply only to $group, your pipeline is empty"
+            )
 
         try:
-            acc = accumulator_dict[accumulator]
+            field_type, acc = aggregation_mapping[accumulator]
         except KeyError as error:
             raise AnnotationError(f"Invalid accumulator: {accumulator}") from error
 
         if isinstance(f, F):
             value = f.to_dict()
         else:
-            value = f"${f}"
+            if isinstance(f, str):
+                try:
+                    self.get_model_field(self.base_model, f)
+                    value = f"${f}"
+                except InvalidField:
+                    value = f
+            else:
+                value = f
+
+        # Determine the data type based on the aggregation operator
         self.pipelines[-1]["$group"] |= {annotate_name: {acc: value}}
         self.base_model._fields[annotate_name] = field_type  # noqa
         return self
@@ -345,66 +440,96 @@ class Aggify:
 
         return merged_pipeline
 
+    @last_out_stage_check
     def lookup(
-        self, from_collection: Document, let: list[str], query: list[Q], as_name: str
+        self,
+        from_collection: Document,
+        as_name: str,
+        query: list[Q] | Q | None = None,
+        let: list[str] | None = None,
+        local_field: str | None = None,
+        foreign_field: str | None = None,
     ) -> "Aggify":
         """
         Generates a MongoDB lookup pipeline stage.
 
         Args:
-            from_collection (Document): The name of the collection to lookup.
-            let (list): The local field(s) to join on.
-            query (list[Q]): List of desired queries with Q function.
+            from_collection (Document): The document representing the collection to perform the lookup on.
             as_name (str): The name of the new field to create.
+            query (list[Q] | Q | None, optional): List of desired queries with Q function or a single query.
+            let (list[str] | None, optional): The local field(s) to join on. If provided, localField and foreignField are not used.
+            local_field (str | None, optional): The local field to join on when let is not provided.
+            foreign_field (str | None, optional): The foreign field to join on when let is not provided.
 
         Returns:
-            Aggify: A MongoDB lookup pipeline stage.
+            Aggify: An instance of the Aggify class representing a MongoDB lookup pipeline stage.
         """
-        check_fields_exist(self.base_model, let)  # noqa
-
-        let_dict = {
-            field: f"${self.base_model._fields[field].db_field}" for field in let
-        }  # noqa
-        from_collection = from_collection._meta.get("collection")  # noqa
 
         lookup_stages = []
+        check_field_exists(self.base_model, as_name)
+        from_collection_name = from_collection._meta.get("collection")  # noqa
 
-        for q in query:
-            # Construct the match stage for each query
-            if isinstance(q, Q):
-                replaced_values = replace_values_recursive(
-                    convert_match_query(dict(q)),  # noqa
-                    {field: f"$${field}" for field in let},
-                )
-                match_stage = {"$match": {"$expr": replaced_values.get("$match")}}
-                lookup_stages.append(match_stage)
-            elif isinstance(q, Aggify):
-                lookup_stages.extend(
-                    replace_values_recursive(
-                        convert_match_query(q.pipelines),  # noqa
+        if not let and not (local_field and foreign_field):
+            raise InvalidArgument(
+                expected_list=[["local_field", "foreign_field"], "let"]
+            )
+        elif not let:
+            if not (local_field and foreign_field):
+                raise InvalidArgument(expected_list=["local_field", "foreign_field"])
+            lookup_stage = {
+                "$lookup": {
+                    "from": from_collection_name,
+                    "localField": get_db_field(self.base_model, local_field),  # noqa
+                    "foreignField": get_db_field(
+                        from_collection, foreign_field
+                    ),  # noqa
+                    "as": as_name,
+                }
+            }
+        else:
+            if not query:
+                raise InvalidArgument(expected_list=["query"])
+            check_fields_exist(self.base_model, let)  # noqa
+            let_dict = {
+                field: f"${get_db_field(self.base_model, field)}"
+                for field in let  # noqa
+            }
+            for q in query:
+                # Construct the match stage for each query
+                if isinstance(q, Q):
+                    replaced_values = replace_values_recursive(
+                        convert_match_query(dict(q)),
                         {field: f"$${field}" for field in let},
                     )
-                )
+                    match_stage = {"$match": {"$expr": replaced_values.get("$match")}}
+                    lookup_stages.append(match_stage)
+                elif isinstance(q, Aggify):
+                    lookup_stages.extend(
+                        replace_values_recursive(
+                            convert_match_query(q.pipelines),  # noqa
+                            {field: f"$${field}" for field in let},
+                        )
+                    )
 
-        # Append the lookup stage with multiple match stages to the pipeline
-        lookup_stage = {
-            "$lookup": {
-                "from": from_collection,
-                "let": let_dict,
-                "pipeline": lookup_stages,  # List of match stages
-                "as": as_name,
+            # Append the lookup stage with multiple match stages to the pipeline
+            lookup_stage = {
+                "$lookup": {
+                    "from": from_collection_name,
+                    "let": let_dict,
+                    "pipeline": lookup_stages,  # List of match stages
+                    "as": as_name,
+                }
             }
-        }
 
         self.pipelines.append(lookup_stage)
 
         # Add this new field to base model fields, which we can use it in the next stages.
-        self.base_model._fields[as_name] = fields.StringField()  # noqa
+        self.base_model._fields[as_name] = from_collection  # noqa
 
         return self
 
     @staticmethod
-    def get_model_field(model: Document, field: str) -> fields:
+    def get_model_field(model: Type[Document], field: str) -> fields:
         """
         Get the field definition of a specified field in a MongoDB model.
 
@@ -425,25 +550,30 @@ class Aggify:
 
     def _replace_base(self, embedded_field) -> str:
         """
-           Replace the root document with a specified embedded field.
+        Replace the root document with a specified embedded field.
 
-           Args:
-               embedded_field (str): The name of the embedded field to use as the new root.
+        Args:
+            embedded_field (str): The name of the embedded field to use as the new root.
 
-           Returns:
-               str: The MongoDB aggregation expression for replacing the root.
+        Returns:
+            str: The MongoDB aggregation expression for replacing the root.
 
-           Raises:
-               InvalidEmbeddedField: If the specified embedded field is not found or is not of the correct type.
+        Raises:
+            InvalidEmbeddedField: If the specified embedded field is not found or is not of the correct type.
         """
-        model_field = self.get_model_field(self.base_model, embedded_field)  # noqa
+        model_field = self.get_model_field(self.base_model, embedded_field)
 
-        if not hasattr(model_field, 'document_type') or not issubclass(model_field.document_type, EmbeddedDocument):
+        if not hasattr(model_field, "document_type") or not issubclass(
+            model_field.document_type, EmbeddedDocument
+        ):
             raise InvalidEmbeddedField(field=embedded_field)
 
         return f"${model_field.db_field}"
 
-    def replace_root(self, *, embedded_field: str, merge: dict | None = None) -> "Aggify":
+    @last_out_stage_check
+    def replace_root(
+        self, *, embedded_field: str, merge: dict | None = None
+    ) -> "Aggify":
         """
         Replace the root document in the aggregation pipeline with a specified embedded field or a merged result.
 
@@ -461,26 +591,17 @@ class Aggify:
         name = self._replace_base(embedded_field)
 
         if not merge:
-            new_root = {
-                "$replaceRoot": {
-                    "$newRoot": name
-                }
-            }
+            new_root = {"$replaceRoot": {"$newRoot": name}}
         else:
-            new_root = {
-                '$replaceRoot': {
-                    'newRoot': {
-                        '$mergeObjects': [
-                            merge, name
-                        ]
-                    }
-                }
-            }
+            new_root = {"$replaceRoot": {"newRoot": {"$mergeObjects": [merge, name]}}}
         self.pipelines.append(new_root)
 
         return self
 
-    def replace_with(self, *, embedded_field: str, merge: dict | None = None) -> "Aggify":
+    @last_out_stage_check
+    def replace_with(
+        self, *, embedded_field: str, merge: dict | None = None
+    ) -> "Aggify":
         """
         Replace the root document in the aggregation pipeline with a specified embedded field or a merged result.
 
@@ -498,17 +619,9 @@ class Aggify:
         name = self._replace_base(embedded_field)
 
         if not merge:
-            new_root = {
-                "$replaceWith": name
-            }
+            new_root = {"$replaceWith": name}
         else:
-            new_root = {
-                '$replaceWith': {
-                    '$mergeObjects': [
-                        merge, name
-                    ]
-                }
-            }
+            new_root = {"$replaceWith": {"$mergeObjects": [merge, name]}}
         self.pipelines.append(new_root)
 
         return self
